@@ -44,48 +44,6 @@ def l2norm_tensors(*tensors):
     tensors = tuple(map(lambda t: t.type(dtype), tensors))
     return tensors
 
-# decorators
-
-def merged_batch_head_queries(fn):
-    @wraps(fn)
-    def inner(
-        q,
-        k,
-        v,
-        mask = None,
-        attn_bias = None,
-        scale = 10,
-        causal = False,
-        l2norm_qk = True,
-        attn_bias_batch_dim = False
-    ):
-        is_merged_batch_heads_query = q.ndim == 3
-
-        if is_merged_batch_heads_query:
-            assert k.ndim == 3 and v.ndim ==3, 'if batch and heads are merged for queries, keys and values must also similarly have only 3 dimensions'
-
-            attn_bias_batch_dim = True
-            q = q[:, None, ...]
-
-        out = fn(
-            q,
-            k,
-            v,
-            mask = mask,
-            attn_bias = attn_bias,
-            scale = scale,
-            causal = causal,
-            l2norm_qk = l2norm_qk,
-            attn_bias_batch_dim = attn_bias_batch_dim
-        )
-
-        if is_merged_batch_heads_query:
-            out = out.squeeze(1)
-
-        return out
-
-    return inner
-
 # original cosine sim attention
 
 # b - batch
@@ -94,7 +52,6 @@ def merged_batch_head_queries(fn):
 # j - target sequence length
 # d - feature dimension
 
-@merged_batch_head_queries
 def plain_cosine_sim_attention(
     q,
     k,
@@ -108,8 +65,16 @@ def plain_cosine_sim_attention(
 
 ):
     assert not (causal and exists(mask)), 'mask should not be supplied if causality is needed'
+
+    is_merged_batch_heads_query = q.ndim == 3
     single_head_kv = k.ndim == 3
 
+    if is_merged_batch_heads_query:
+        assert k.ndim == 3 and v.ndim ==3, 'if batch and heads are merged for queries, keys and values must also similarly have only 3 dimensions'
+
+        attn_bias_batch_dim = True
+        q = q[:, None, ...]
+
     if l2norm_qk:
         q, k = l2norm_tensors(q, k)
 
@@ -131,7 +96,12 @@ def plain_cosine_sim_attention(
         sim = sim.masked_fill(~mask[:, None, None, :], mask_value)
 
     attn = sim.softmax(dim = -1)
-    return einsum(f'b h i j, {kv_einsum_eq} -> b h i d', attn, v)
+    out = einsum(f'b h i j, {kv_einsum_eq} -> b h i d', attn, v)
+
+    if is_merged_batch_heads_query:
+        out = out.squeeze(1)
+
+    return out
 
 # main class
 
@@ -198,7 +168,6 @@ flash_cosine_sim_attention_cuda = FlashCosineSimAttention.apply
 
 # wrapper function
 
-@merged_batch_head_queries
 def flash_cosine_sim_attention(
     q,
     k,
